import React, { useState, useRef, useEffect } from 'react';
import {
  Button,
  DatePicker,
  Input,
  Select,
  Space,
  Table,
  message,
  Modal,
} from 'antd';
import styles from './index.less';
import ContractModal, { Type, Status } from './component/Modal';
import { PageContainer } from '@ant-design/pro-components';
import { useRequest, useModel } from '@umijs/max';
import { connect } from 'umi';
import {
  queryApproval,
  queryCompany,
  queryContract,
  queryContractCancel,
  queryContractCancelCheck,
  queryContractCheck,
  queryContractDownload,
  queryDelById,
  queryGetContractList,
  queryOAReCall,
} from '../../services/contract';
import dayjs from 'dayjs';
import FileViewerModal from '@/components/FileViewerNew';
import { getToken } from '@/utils/utils';
import PageContent from '@/components/PageContent';
import EllipsisText from './component/EllipsisText';
import { stringify } from 'qs';
import { audit, createAduit } from '@/services/boom';
import { ExclamationCircleOutlined } from '@ant-design/icons';

const ConteactManager = (props) => {
  const { dispatch } = props;
  const [searchData, setSearchData] = useState({
    effect_on: '',
    project_name: '',
    status: '',
    page_size: 10,
    current: 1,
    name: '',
  });
  const {
    initialState: { user },
  } = useModel('@@initialState');
  const [visible, setVisible] = useState(false);
  const [detail, setDetail] = useState({});
  const [data, setData] = useState([]);
  const [pagination, setPagination] = useState({ current: 1 });
  const typeRef = useRef();
  const parentIdRef = useRef(0);
  const [fileViewerVisible, setFileViewerVisible] = useState(false);
  const [fileViewerData, setFileViewerData] = useState();
  const [modal, contextHolder] = Modal.useModal();

  const showBtn = (record, type) => {
    let bool = false;
    switch (type) {
      case 'download':
        if (user?.Permission['menu-001-audit'] || record.created_by == user.ID)
          bool = true;
        break;
      // case 'addOrCal':
      //   if (record.created_by == user.ID) bool = true;
      //   break;
    }
    return bool;
  };

  const columns = [
    {
      title: '合同编号',
      dataIndex: 'code',
      key: 'code',
      align: 'center',
      width: 160,
    },
    {
      title: '合同签订时间',
      dataIndex: 'effect_on',
      key: 'effect_on',
      align: 'center',
      width: 120,
    },
    {
      title: '合同名称',
      dataIndex: 'name',
      key: 'name',
      align: 'center',
      width: 100,
    },
    {
      title: '甲方',
      dataIndex: 'party_a',
      key: 'party_a',
      align: 'center',
      render: (text) => <EllipsisText text={text} width={120} />,
    },
    {
      title: '乙方',
      dataIndex: 'party_b',
      key: 'party_b',
      align: 'center',
      render: (text) => <EllipsisText text={text} width={120} />,
    },
    {
      title: '丙方',
      dataIndex: 'party_c',
      key: 'party_c',
      align: 'center',
      render: (text) => {
        let str = text;
        try {
          str = JSON.parse(text).join('、');
        } catch (error) {}
        return <EllipsisText text={str} width={120} />;
      },
    },
    {
      title: '所属部门/子公司',
      dataIndex: 'dep_name',
      key: 'dep_name',
      align: 'center',
      width: 160,
    },
    {
      title: '项目名称',
      dataIndex: 'project_name',
      key: 'project_name',
      align: 'center',
      width: 120,
    },
    {
      title: '合同总价（万元）',
      dataIndex: 'amount',
      key: 'amount',
      align: 'center',
      width: 160,
    },
    {
      title: '经办人',
      dataIndex: 'deal_by',
      key: 'deal_by',
      align: 'center',
      width: 80,
    },
    {
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      align: 'center',
      width: 100,
      render: (status) => {
        let str = '';
        let color = 'black';
        switch (status) {
          case Status.ReCall:
            str = '待提交';
            break;
          case Status.Checking:
            str = '待审核';
            color = 'blue';
            break;
          case Status.CheckReject:
            str = '审核拒绝';
            color = 'red';
            break;
          case Status.CheckSuccess:
            str = '已存档';
            color = 'green';
            break;
          case Status.CalChecking:
            str = '作废待审核';
            color = 'blue';
            break;
          case Status.CalCheckReject:
            str = '作废拒绝';
            color = 'red';
            break;
          case Status.CalCheckSuccess:
            str = '已作废';
            break;
        }
        return <div style={{ color }}>{str}</div>;
      },
    },
    {
      title: '操作',
      width: '210px',
      align: 'center',
      render: (record) => {
        return (
          <Space>
            <a
              onClick={() => {
                typeRef.current = Type.detail;
                parentIdRef.current = 0;
                setDetail(record);
                setVisible(true);
              }}
            >
              详情
            </a>
            <a onClick={() => handlePreView(record)}>预览</a>
            {showBtn(record, 'download') && (
              <a onClick={() => handleUpload(record)}>下载</a>
            )}
            {record.status == Status.CheckSuccess && (
              <a
                onClick={() => {
                  typeRef.current = Type.cancel;
                  setDetail(record);
                  setVisible(true);
                }}
              >
                作废
              </a>
            )}
            {(record.status == Status.ReCall ||
              record.status == Status.CheckReject) && (
              <a onClick={() => handleDelete(record.id)}>删除</a>
            )}
          </Space>
        );
      },
    },
  ];

  const config = {
    title: '添加成功！',
    content: (
      <>
        将合同（含附件）的原件和相关资料交给集团档案管理部门或分子公司合同专员才能完成合同存档，请注意及时存档。
      </>
    ),
  };

  useEffect(() => {
    dispatch({
      type: 'user/fetch',
    });
  }, []);

  //请求列表
  const { run, loading } = useRequest((data) => queryGetContractList(data), {
    defaultParams: [searchData],
    onSuccess: (data) => {
      let resultData = data?.list?.map((item) => {
        return item.sub_num > 0 ? { ...item, children: [] } : item;
      });

      setData(resultData);
      setPagination(data?.pagination);
    },
  });
  //编辑新增接口
  const { run: editRun } = useRequest((data) => queryContract(data), {
    manual: true,
    onSuccess: () => {
      // message.success('添加成功');
      setVisible(false);
      run(searchData);
      modal.info(config);
    },
    onError: () => {
      message.success('添加失败');
    },
  });

  //作废发起
  const { run: calRun } = useRequest((data) => queryContractCancel(data), {
    manual: true,
    onSuccess: () => {
      message.success('发起作废成功');
      setVisible(false);
      run(searchData);
    },
    onError: () => {
      message.success('发起作废失败');
    },
  });

  // //作废审核
  // const { run: calCheckRun } = useRequest(
  //   (data) => queryContractCancelCheck(data),
  //   {
  //     manual: true,
  //     onSuccess: () => {
  //       message.success('审核成功');
  //       setVisible(false);
  //       run(searchData);
  //     },
  //     onError: () => {
  //       message.success('审核失败');
  //     },
  //   },
  // );

  // //审核合同
  // const { run: runCheck, loading: checkLoading } = useRequest(
  //   (data) => queryContractCheck(data),
  //   {
  //     manual: true,
  //     onSuccess: () => {
  //       // conAuditRun({ multi_status: '1,4', currentPage: 1, page_size: 10 });
  //       // conAuditedRun({ check_by: user.CName });
  //       setVisible(false);
  //       run(searchData);
  //       message.success('审核成功');
  //     },
  //     onErroe: () => {
  //       message.error('审核失败');
  //     },
  //   },
  // );

  //审核合同
  const { run: runOACheck } = useRequest(audit, {
    manual: true,
    onSuccess: (data) => {
      if (data?.Code) {
        message.error(data?.Msg || '审核失败');
        return;
      }
      setTimeout(() => {
        run(searchData);
      }, 1000);
      setVisible(false);
      message.success('审核成功');
    },
    onErroe: () => {
      message.error('审核失败');
    },
  });

  //请求项目列表
  const { data: projectData } = useRequest(queryApproval, {
    defaultParams: [{ pageSize: 99999 }],
  });

  //发起OA审批
  const { run: createRun } = useRequest(
    (data) => createAduit({ ...data, flow_id: 40, cc_list: [], files: '' }),
    {
      manual: true,
    },
  );

  const handleDelete = (id) => {
    modal.confirm({
      icon: <ExclamationCircleOutlined />,
      title: '提示：',
      content: <div>确定删除合同，删除后不可回复！</div>,
      onOk: async () => {
        const res = await queryDelById({ id });
        if (res.code == 200) {
          message.success('删除成功');
          run(searchData);
        } else {
          message.success('删除失败，请重试');
        }
      },
    });
  };
  const handlePreView = (data) => {
    if (!data?.attach && !data?.attach_extend) return;
    const result = [];
    if (data?.attach) {
      const attach = JSON.parse(data?.attach);
      attach.forEach((item) => {
        result.push({ ...item, type: '合同及合同附件' });
      });
    }
    if (data?.attach_extend) {
      const attach_extend = JSON.parse(data?.attach_extend);
      attach_extend.forEach((item) => {
        result.push({ ...item, type: '合同相关资料' });
      });
    }
    setFileViewerData(result);
    setFileViewerVisible(true);
  };

  //单个合同下载
  const handleUpload = (record) => {
    const token = getToken();
    window.downloadFile(
      `/api/contract/v1/contract/download?id=${record.id}&JWT-TOKEN=${token}`,
      record.name,
      false,
    );
  };
  const handleSearch = () => {
    run(searchData);
  };
  //合同列表导出列表下载
  const handleExport = async () => {
    const token = getToken();
    const params = { ...searchData, current_user: user.ID };
    window.downloadFile(
      `/api/contract/v1/contract-list/download?${stringify(
        params,
      )}&JWT-TOKEN=${token}`,
      '合同列表.xlsx',
      false,
    );
  };

  const handleQueryChildren = async (req) => {
    const res = await queryGetContractList(req);
    if (res?.data?.list) {
      let resultData = [...data];
      let idx = data.findIndex((item) => item.id == req.is_parent);
      if (idx > -1) {
        resultData[idx].children = res?.data?.list;
        setData(resultData);
        setPagination(res.data?.pagination);
        // setData({ list: resultData, pagination: res.data?.pagination });
      }
    }
  };

  const handleOk = (data, type, formData = [], audit_list = []) => {
    if (type == Type.add) {
      editRun(data);
      createRun({
        form: JSON.stringify(formData),
        audit_list,
        extend_code: data.code,
        extend_type: 0, //归档提交
      });
    } else if (type == Type.cancel) {
      calRun(data);
      createRun({
        form: JSON.stringify(formData),
        audit_list,
        extend_code: data.code,
        extend_type: 1, //作废提交
      });
    } else if (type == Type.check) {
      runOACheck(data);
    }
  };

  const handlerReCall = async (id) => {
    modal.confirm({
      icon: <ExclamationCircleOutlined />,
      title: '提示：',
      content: <div>确定撤回合同存档审批！</div>,
      onOk: async () => {
        const res = await queryOAReCall(id);
        if (res.code == 200) {
          message.success('撤回成功');
          setVisible(false);
          run(searchData);
        } else {
          message.success('撤回失败，请重试');
        }
      },
    });
  };

  const onPageChange = (page) => {
    run({ ...searchData, current: page });
  };
  return (
    <PageContent>
      <div className={styles.searchContent}>
        <div className={styles.itemFlex}>
          <div>合同签订日期：</div>
          <DatePicker
            onChange={(e) => {
              setSearchData({
                ...searchData,
                effect_on: e ? dayjs(e).format('YYYY-MM-DD') : null,
              });
            }}
          />
        </div>
        <div className={styles.itemFlex}>
          <div>项目名称：</div>
          <Select
            style={{ width: 200 }}
            placeholder="请选择"
            onChange={(e) => {
              setSearchData({
                ...searchData,
                project_name: e,
              });
            }}
            options={projectData?.list?.map((item) => {
              return {
                value: item.project_name,
                label: item.project_name,
              };
            })}
          />
        </div>
        <div className={styles.itemFlex}>
          <div>状态：</div>
          <Select
            style={{ width: 150 }}
            placeholder="请选择"
            allowClear
            onChange={(e) => {
              setSearchData({
                ...searchData,
                status: e,
              });
            }}
            options={[
              { value: 3, label: '已归档' },
              { value: 1, label: '归档审核中' },
              { value: 4, label: '作废审核中' },
              { value: 6, label: '已作废' },
              { value: 2, label: '归档拒绝' },
              { value: 5, label: '作废拒绝' },
            ]}
          />
        </div>
        <Input
          style={{ width: 150 }}
          className={styles.inputSty}
          placeholder="请输入合同名称/编号"
          allowClear
          onChange={(e) => {
            setSearchData({
              ...searchData,
              name: e.target.value,
            });
          }}
        />
        <Button
          type="primary"
          className={styles.searchBtnSty}
          onClick={handleSearch}
        >
          查询
        </Button>
        <Button
          type="primary"
          onClick={() => {
            typeRef.current = Type.add;
            setDetail({});
            setVisible(true);
          }}
        >
          新增
        </Button>
        <Button
          type="primary"
          className={styles.exportBtnSty}
          onClick={handleExport}
        >
          导出
        </Button>
      </div>
      <Table
        rowKey="code"
        loading={loading}
        columns={columns}
        dataSource={data}
        indentSize={70}
        onExpand={(expanded, record) => {
          console.log(expanded, record);
          if (expanded) handleQueryChildren({ is_parent: record.id });
        }}
        pagination={{ ...pagination, onChange: onPageChange }}
      />
      <ContractModal
        detail={detail}
        type={typeRef.current}
        parent_id={parentIdRef.current}
        projectList={projectData?.list}
        visible={visible}
        handleOk={handleOk}
        handleCancel={() => setVisible(false)}
        handlerReCall={handlerReCall}
      />
      <FileViewerModal
        data={fileViewerData}
        visible={fileViewerVisible}
        downloadFile={() => {}}
        onCancel={() => {
          setFileViewerVisible(false);
        }}
      />
      {contextHolder}
    </PageContent>
  );
};
export default connect(({ user, loading }) => ({
  userList: user.list,
}))(ConteactManager);
